#ifndef GROUNDER_GROUNDERS_FAST_DOWNWARD_GROUNDER_H_
#define GROUNDER_GROUNDERS_FAST_DOWNWARD_GROUNDER_H_

#include "grounder.h"

#include "../rule_matcher.h"

#include <iostream>
#include <optional>

#include <absl/container/flat_hash_set.h>


class FastDownwardGrounder : public Grounder {
    bool is_new(Fact &new_fact,
                absl::flat_hash_set<Fact> &reached_facts,
                LogicProgram &lp);

protected:
    RuleMatcher rule_matcher;

    void create_rule_matcher(const LogicProgram &lp) {
        // Loop over rule conditions
        for (const auto &rule : lp.get_rules()) {
            int cont = 0;
            for (const auto &condition : rule->get_conditions()) {
                rule_matcher.insert(condition.get_predicate_index(),
                                    rule->get_index(),
                                    cont++);
            }
        }
    }

    void project(const RuleBase &rule, const Fact &fact, std::vector<Fact>& newfacts);
    void join(RuleBase &rule, const Fact &fact, int position, std::vector<Fact>& newfacts);
    void product(RuleBase &rule, const Fact &fact, int position, std::vector<Fact>& newfacts);

public:
    FastDownwardGrounder() {};

    int ground(LogicProgram &lp) override;

};

#endif //GROUNDER_GROUNDERS_FAST_DOWNWARD_GROUNDER_H_
